@@ -37,7 +37,11 @@
    get-method-name
    suitable-for-method?
    process-explicit-names
-   gi-make-callback-entry))
+   gi-make-callback-entry
+   parse-ignored-slots
+   my-get-name5
+   dp
+   dpl))
 
 
 ;; (eval-when (expand load eval)
@@ -76,6 +80,13 @@
 (use-modules (th-scheme-utilities stdutils))
 (import (except (rnrs base) map))
 
+
+;; (define dp display)
+;; (define (dpl x) (display x) (newline))
+(define (dp x) #f)
+(define (dpl x) #f)
+
+
 (define-class <golf-generator> ()
   (l-rejected-functions #:init-keyword #:l-rejected-functions)
   (l-rejected-methods #:init-keyword #:l-rejected-methods)
@@ -89,11 +100,14 @@
   (l-imported-only-type #:init-value '())
   (l-imported-functions #:init-value '())
   (l-toplevel-functions #:init-value '())
+  (al-ignored-slots #:init-keyword #:al-ignored-slots #:init-value '())
   (al-callbacks #:init-value '())
   (al-versions #:init-keyword #:al-versions)
   (debug? #:init-keyword #:debug?)
   (generate-methods? #:init-keyword #:generate-methods?)
-  (check-procedures? #:init-keyword #:check-procedures?))
+  (check-procedures? #:init-keyword #:check-procedures?)
+  (generate-accessors? #:init-keyword #:generate-accessors?
+                       #:init-value #f))
 
 (define-class <intr-entity> ()
   (str-namespace #:init-keyword #:str-namespace)
@@ -132,9 +146,12 @@
 
 (define-class <intr-interface> (<intr-type>))
 
-(define-class <intr-enum> (<intr-type>))
+(define-class <intr-enum> (<intr-type>)
+  (al-values #:init-keyword #:al-values))
 
-(define-class <intr-flags> (<intr-type>))
+(define-class <intr-flags> (<intr-type>)
+  (al-values #:init-keyword #:al-values)
+  (s-component-type #:init-keyword #:s-component-type))
 
 (define-class <intr-struct> (<intr-type>)
   (is-opaque? #:init-keyword #:is-opaque? #:init-value #f)
@@ -208,6 +225,32 @@
      ">")))
 
 
+(define (my-get-name4 info)
+  (my-get-name2
+   (g-base-info-get-namespace info)
+   (g-base-info-get-name info)))
+
+
+(define (my-get-name5 s-namespace s-name)
+  (string-append
+   "<"
+   (g-name->name (symbol->string s-namespace) #t)
+   "-"
+   (g-name->name (symbol->string s-name) #t)
+   ">"))
+
+
+(define (get-enum-name enum-type)
+  (strong-assert (or (is-a? enum-type <gi-enum>)
+                     (is-a? enum-type <gi-flags>)))
+  (let ((str-name (slot-ref enum-type 'g-name)))
+    (string->symbol
+      (string-append
+       "<"
+       (g-studly-caps-expand str-name)
+       ">"))))
+
+
 (define (get-golf-imports str-filename)
   (assert (string? str-filename))
   (let ((ip (open-input-file str-filename)))
@@ -246,6 +289,34 @@
     al-versions))
 
 
+(define (parse-ignored-slots l-data)
+  (let ((al-ignored-slots '()))
+    (for-each
+     (lambda (l-entry)
+       (if (and (list? l-entry)
+                (not-null? l-entry)
+                (eq? (car l-entry) 'ignore-slots))
+         (if (and (>= (length l-entry) 2)
+                  (list? (cadr l-entry))
+                  (eqv? (length (cadr l-entry)) 2)
+                  (for-all symbol? (cadr l-entry)))
+           (let* ((s-namespace (caadr l-entry))
+                  (s-class-name (cadr (cadr l-entry)))
+                  (str-class-name (my-get-name5 s-namespace s-class-name))
+                  (s-class-name (string->symbol str-class-name))
+                  (l-slots (cddr l-entry)))
+             (if (and (list? l-slots) (for-all symbol? l-slots))
+               (set! al-ignored-slots
+                 (cons (cons s-class-name l-slots)
+                       al-ignored-slots))
+               (raise (list 'ignore-slots:invalid-slots
+                            (cons 'l-entry l-entry)))))
+           (raise (list 'ignore-slots:syntax-error
+                        (cons 'l-entry l-entry))))))
+     l-data)
+    al-ignored-slots))
+
+
 (define (make-target-class-name s-namespace s-class-name)
   (if s-namespace
     (let* ((str-namespace (g-studly-caps-expand (symbol->string s-namespace)))
@@ -262,6 +333,13 @@
       s-final-name)))
 
 
+(define-method (search-entity (generator <golf-generator>)
+                              (s-target-name <symbol>))
+  (let ((l-all-entities (slot-ref generator 'l-all-entities)))
+    (find (lambda (ent) (eq? (slot-ref ent 's-target-name) s-target-name))
+          l-all-entities)))
+
+
 (define (gi-import-info1 info generator toplevel? methods?)
   (assert (is-a? generator <golf-generator>))
   (assert (boolean? toplevel?))
@@ -301,9 +379,8 @@
          (push! i-type %gi-imported-base-info-types1))
        
        ;; TBR
-       ;; (display "class: ")
-       ;; (display (g-base-info-get-name info))
-       ;; (newline)
+       (dp "class: ")
+       (dpl (g-base-info-get-name info))
        
        (gi-import-object1 info generator toplevel? methods?))
       ((interface)
@@ -372,7 +449,7 @@
   (assert (is-a? generator <golf-generator>))
   
   ;; TBR
-  ;; (display "gi-import-object1 ENTER\n")
+  (dpl "gi-import-object1 ENTER")
   ;; (display methods?)
   ;; (newline)
   
@@ -485,7 +562,7 @@
   (assert (is-a? generator <golf-generator>))
   
   ;; TBR
-  ;; (display "g-object-import1\n")
+  (dpl "g-object-import1")
   
   (match parent
          ((p-info p-namespace p-name)
@@ -501,7 +578,7 @@
   (assert (is-a? generator <golf-generator>))
   
   ;; TBR
-  ;; (display "g-object-import-with-supers1\n")
+  (dpl "g-object-import-with-supers1")
   
   (match child
          ((info namespace name)
@@ -517,8 +594,16 @@
                  (gi-name (g-type-name r-type))
                  (str-basename (g-studly-caps-expand gi-name))
                  (s-target-name (g-name->class-name gi-name)))
+
+            ;; TBR
+            (if (eq? s-target-name '<gtk-text-tag>)
+              (begin
+               (dpl "<gtk-text-tag> HEP")
+               (dpl methods?)))
+
             (cond
               ((not (member gi-name (slot-ref generator 'l-imported-types)))
+               (dpl "import/1")
                (let* ((str-name (g-base-info-get-name info))
                       (str-namespace (g-base-info-get-namespace info))
                       (intr-object
@@ -529,6 +614,7 @@
                              #:l-methods '()
                              #:l-supers supers
                              #:l-properties '())))
+                 
                  (add-entity generator intr-object)
                  (add-imported-type generator gi-name)
                  
@@ -550,32 +636,34 @@
                     (gi-import-methods1 info intr-object generator
                                         g-object-info-get-n-methods
                                         g-object-info-get-method
-                                        str-basename)
+                                        str-basename s-target-name)
                     (gi-import-properties1 info intr-object generator
                                            str-basename
                                            s-target-name))
                    (add-imported-only-type generator gi-name))))
               ((and methods?
                     (member gi-name (slot-ref generator 'l-imported-only-type)))
+               (dpl "import/2")
                (let* ((str-name (g-base-info-get-name info))
                       (str-namespace (g-base-info-get-namespace info))
-                      (intr-object
-                       (make <intr-object>
-                             #:str-namespace str-namespace
-                             #:str-name str-name
-                             #:s-target-name s-target-name
-                             #:l-methods '()
-                             #:l-supers supers
-                             #:l-properties '())))
+                      (intr-object (search-entity generator s-target-name)))
+                 (assert (is-a? intr-object <intr-entity>))
+                 (dpl "import/2-1")
                  (remove-type1 generator gi-name)
+                 (dpl "import/2-2")
                  (add-imported-type generator gi-name)
+                 (dpl "import/2-3")
                  (gi-import-methods1 info intr-object generator
                                      g-object-info-get-n-methods
                                      g-object-info-get-method
-                                     str-basename)
+                                     str-basename s-target-name)
+                 (dpl "import/2-4")
                  (gi-import-properties1 info intr-object generator
                                         str-basename
-                                        s-target-name))))))))
+                                        s-target-name)))
+              (else
+               (dpl "import/3")
+               #f))))))
 
 
 (define (gi-import-nonobj-type info generator toplevel? methods?
@@ -623,7 +711,7 @@
            (gi-import-methods1 info intr generator
                                get-n-methods
                                get-method
-                               str-basename)
+                               str-basename s-target-name)
            (add-imported-only-type generator gi-name))))	    
       ((and methods?
             (member gi-name (slot-ref generator 'l-imported-only-type)))
@@ -635,7 +723,7 @@
          (gi-import-methods1 info intr generator
                              get-n-methods
                              get-method
-                             str-basename))))))
+                             str-basename s-target-name))))))
 
 
 (define (gi-import-interface1 info generator toplevel? methods?)
@@ -717,13 +805,20 @@
                        #:str-namespace str-namespace
                        #:str-name str-name
                        #:s-target-name s-target-name
-                       #:l-methods '()))))	   
+                       #:l-methods '()
+                       #:al-values (gi-enum-value-values info)))))
     (gi-import-nonobj-type info generator toplevel? methods?
                            create-intr
                            g-enum-info-get-n-methods
                            g-enum-info-get-method)))
 
 
+(define (get-flags-component-type-name info)
+  (let* ((str-name1 (gi-registered-type-info-name info))
+         (str-name2 (g-name->name str-name1 #t)))
+    (string->symbol (string-append "<_" str-name2 ">"))))
+
+
 (define (gi-import-flags1 info generator toplevel? methods?)
   (let ((create-intr
          (lambda (str-namespace str-name s-target-name)
@@ -731,7 +826,10 @@
                        #:str-namespace str-namespace
                        #:str-name str-name
                        #:s-target-name s-target-name
-                       #:l-methods '()))))	   
+                       #:l-methods '()
+                       #:al-values (gi-enum-value-values info)
+                       #:s-component-type
+                       (get-flags-component-type-name info)))))
     ;; G-Golf uses enum accessors for flags.
     (gi-import-nonobj-type info generator toplevel? methods?
                            create-intr
@@ -741,7 +839,8 @@
 
 (define (gi-import-methods1 info intr-type generator
                             get-n-methods get-method
-                            str-basename)
+                            str-basename s-class-name)
+  (dpl "gi-import-methods1 ENTER")
   (assert (is-a? generator <golf-generator>))
   (assert (procedure? get-n-methods))
   (assert (procedure? get-method))
@@ -751,6 +850,7 @@
     (do ((i 0
             (+ i 1)))
       ((= i n-method))
+      (dpl "gi-import-methods1/1")
       (let* ((m-info (get-method info i))
              (intf-method
               (gi-make-function-entry m-info generator str-basename)))
@@ -763,26 +863,42 @@
         ;    (newline)))
         
         (set! l-methods (cons intf-method l-methods))
+        (dpl "gi-import-methods1/2")
         (add-entity generator intf-method)
+        (dpl "gi-import-methods1/3")
         (add-imported-function generator
                                (slot-ref intf-method 's-target-name))
-        (gi-check-method m-info generator)))
+        (gi-check-method m-info generator s-class-name)))
+    (dpl "gi-import-methods1 EXIT")
     (slot-set! intr-type 'l-methods l-methods)))
 
 
-(define (gi-check-method info generator)
+(define (gi-check-method info generator s-class-name)
+  (dpl "gi-check-method ENTER")
+  (dpl s-class-name)
   (assert (is-a? generator <golf-generator>))
   (let ((return-type (g-callable-info-get-return-type info)))
     (if (eq? (g-type-info-get-tag return-type) 'interface)
-      (gi-import-info1 (g-type-info-get-interface return-type)
-                       generator #f #f)))
+      (let ((s-result-class
+             (my-get-name4
+              (g-type-info-get-interface return-type))))
+        (dpl s-result-class)
+        (if (not (eq? s-result-class s-class-name))
+          (gi-import-info1 (g-type-info-get-interface return-type)
+                           generator #f #f)))))
+  (dpl "gi-check-method/1")
   (let ((n (g-callable-info-get-n-args info)))
     (do ((i 0 (+ i 1))) ((>= i n))
       (let* ((info-arg (g-callable-info-get-arg info i))
              (tp (g-arg-info-get-type info-arg)))
         (if (eq? (g-type-info-get-tag tp) 'interface)
-          (let ((x-intf (g-type-info-get-interface tp)))
-            (gi-import-info1 x-intf generator #f #f)))))))
+          (begin
+           (let* ((x-intf (g-type-info-get-interface tp))
+                  (s-class-name2 (my-get-name4 x-intf)))
+             (dpl "gi-check-method/2")
+             (dpl s-class-name2)
+             (if (not (eq? s-class-name2 s-class-name))
+               (gi-import-info1 x-intf generator #f #f)))))))))
 
 
 (define (gi-import-properties1 info intr-object generator
@@ -793,6 +909,10 @@
   (assert (symbol? s-class-name))
   (let ((i-count (g-object-info-get-n-properties info))
         (l-properties '()))
+    (dp "gi-import-properties1 ")
+    (dpl i-count)
+    (dpl (slot-ref intr-object 's-target-name))
+    (dpl s-class-name)
     (do ((i 0 (+ i 1))) ((>= i i-count))
       (let* ((info-prop (g-object-info-get-property info i))
              (str-prop-name (g-base-info-get-name info-prop))
@@ -829,11 +949,21 @@
                     #:s-getter-name s-getter-name
                     #:s-setter-name s-setter-name
                     #:s-accessor-name s-accessor-name)))
+
+        ;; TBR
+        (if (and  (equal? str-prop-name "show-spaces-set")
+                  (eq? s-property-class '<gtk-text-tag>))
+          (display "tag HEP1\n"))
+        (dp "gi-import-properties1/1 ")
+        (dpl s-property-class)
+
         (if (eq? (g-type-info-get-tag prop-type) 'interface)
           (gi-import-info1 (g-type-info-get-interface prop-type)
                            generator #f #f))
         (add-entity generator intr-property)
         (set! l-properties (cons intr-property l-properties))))
+    (dp "gi-import-properties1 EXIT ")
+    (dpl (length l-properties))
     (slot-set! intr-object 'l-properties l-properties)))
 
 
@@ -855,9 +985,9 @@
                ((eq? (car type-desc) 'gslist)
                 '<list>)
                ((eq? (car type-desc) 'enum)
-                '<symbol>)
+                (get-enum-name (caddr type-desc)))
                ((eq? (car type-desc) 'flags)
-                '(:uniform-list <symbol>))
+                (get-enum-name (caddr type-desc)))
                ((eq? (car type-desc) 'union)
                 ;; '<pointer>)  
                 (my-get-name (!g-name (caddr type-desc))))
@@ -1101,8 +1231,8 @@
                ;;  (newline)
                ;;  (display (slot-ref intr-function 'str-base))
                ;;  (newline)
-               (if (equal? namespace "GLib")
-                 (display "GLib HEP\n"))
+            ;;    (if (equal? namespace "GLib")
+            ;;      (display "GLib HEP\n"))
                
                intr-function))))
 
