@@ -1,5 +1,5 @@
 
-;; Copyright (C) 2016, 2018, 2022 Tommi Höynälänmaa
+;; Copyright (C) 2016, 2018, 2022, 2024 Tommi Höynälänmaa
 
 ;; This file is part of Theme-D-Golf.
 
@@ -18,22 +18,26 @@
 ;; https://www.gnu.org/licenses/.
 
 
-(library (guile-theme-d-intr support)
+(define-module (guile-theme-d-intr support))
+         
+(export theme-gtype-instance-signal-connect slot-set0!)
+         
+(import (srfi srfi-1)
+        (oop goops)
+        (except (g-golf) map)
+        (theme-d runtime rte-base)
+        (theme-d runtime runtime-theme-d-environment)
+        (th-scheme-utilities stdutils))
+
+(define (theme-gtype-instance-signal-connect obj sym-name proc-handler)
+  (connect
+   obj sym-name
+   (lambda args
+     (_i_call-proc proc-handler
+                   args
+                   (map theme-type-of args))))
+  #f)
+
+(define slot-set0! slot-set!)
 
-  (export theme-gtype-instance-signal-connect)
-  
-  (import (guile)
-	  (srfi srfi-1)
-	  (oop goops)
-	  (except (g-golf) map)
-	  (theme-d runtime runtime-theme-d-environment)
-	  (th-scheme-utilities stdutils))
 
-  (define (theme-gtype-instance-signal-connect obj sym-name proc-handler)
-    (connect
-     obj sym-name
-     (lambda args
-       (_i_call-proc proc-handler
-		     args
-		     (map theme-class-of args))))
-    #f))
